import { readFileSync, writeFileSync } from "fs";
import path from "path";
import { encryptFileWLit } from "../src/encryptFile";
import { decryptFileWithContractConditions } from "../src/decryptFile";
import { mintOwnerNFT } from "../src/mintOwnerNFT";
import { fileContractLookup } from "../src/fileContractLookup";
import { mintToAddress } from "../src/mintToAddress";
import { getAllUserTransactions, getTxReceipt } from "../src/findOwnerFiles";
import { EvmContractConditions } from "@lit-protocol/types";
import * as ethers from "ethers";

const chain = "baseSepolia";
const RPC_URL = "https://base-sepolia.g.alchemy.com/v2/L4Rh_Myxql5_39cFaK2wav7aiA12496H";
const contractAddress = "0x5f92Cf56559Fdaa0892a92Ab7d22007791b82686";
const validatorAddress = "0x0F6Af2F14A54A6e6FD95281b82e2578F79A1b476";

// Load the contract ABI
const abiPath = path.join(__dirname, 'contractABI.json');
const contractData = JSON.parse(readFileSync(abiPath, 'utf-8'));
const abi = contractData.abi;

//Load the minting contract ABI
const mintAbiPath = path.join(__dirname, 'mintContractABI.json');
const mintContractData = JSON.parse(readFileSync(mintAbiPath, 'utf-8'));
const mintAbi = mintContractData.abi;

let FILE_TO_ENCRYPT_PATH = "";

// Get the private key from the command line arguments
const ETHEREUM_PRIVATE_KEY = process.argv[2];

/*Determine whether you want to: 
encrypt 
decrypt
mint NFT for the owner
lookup the file contract addr 
mint NFT for a user
*/
const command = process.argv[3];

if (!ETHEREUM_PRIVATE_KEY) {
  throw new Error("❌ Error: ETHEREUM_PRIVATE_KEY argument is missing");
}

// Create ethers.js wallet
const provider = new ethers.providers.JsonRpcProvider(RPC_URL);
const wallet = new ethers.Wallet(ETHEREUM_PRIVATE_KEY, provider);

// If we're encrypting, get the path of the file to encrypt and encrypt it
if (command == "encrypt") {
  FILE_TO_ENCRYPT_PATH = path.join(__dirname, process.argv[4]);
  //encryptFile(wallet, FILE_TO_ENCRYPT_PATH);
  encryptFileWLit(
    wallet,
    FILE_TO_ENCRYPT_PATH,
    contractAddress,
    validatorAddress,
    abi,
    chain
  )
}

else if (command == "decrypt") {
  (async () => {
  //load in encrypted file, hash and conditions
  const ciphertextPath = path.join(__dirname, process.argv[4]);
  const dataToEncryptHashPath = path.join(__dirname, process.argv[5]);
  const evmContractConditionsPath = path.join(__dirname, process.argv[6]);
  
  const ciphertext = readFileSync(ciphertextPath, "utf-8");
  const dataToEncryptHash = readFileSync(dataToEncryptHashPath, "utf-8");

  //contract conditions need to be converted from type string to type EvmContractConditions
  const evmContractConditionsString = readFileSync(evmContractConditionsPath, "utf-8");
  const evmContractConditions: EvmContractConditions = JSON.parse(evmContractConditionsString);
  
  await decryptFileWithContractConditions(
    ciphertext,
    dataToEncryptHash,
    wallet,
    chain,
    evmContractConditions as EvmContractConditions
  );})();
}

else if(command == "fileLookup") {
  //read in file identifier
  const fileIdentifierPath = path.join(__dirname, process.argv[4]);
  const fileIdentifier = readFileSync(fileIdentifierPath, "utf-8");

  //find smart contract
  const fileSmartContract = fileContractLookup(
    wallet,
    contractAddress,
    abi,
    fileIdentifier
  )
}

else if(command == "mintOwnerNFT") {
  // Ensure this is in an async context
  (async () => {
    //grab contract address
    const fileIdentifierPath = path.join(__dirname, process.argv[4]);
    const fileIdentifier = readFileSync(fileIdentifierPath, "utf-8");

    //find smart contract
    const fileSmartContract = await fileContractLookup(
      wallet,
      contractAddress,
      abi,
      fileIdentifier
    );

    //mint NFT for owner
    mintOwnerNFT(
      wallet,
      fileSmartContract,
      mintAbi
    );
  })();
}

else if(command == "mintToAddress") {
  // Ensure this is in an async context
  (async () => {
    //grab contract address
    const fileIdentifierPath = path.join(__dirname, process.argv[4]);
    const fileIdentifier = readFileSync(fileIdentifierPath, "utf-8");

    //grab destination wallet
    const destinationWallet = process.argv[5];

    //find smart contract
    const fileSmartContract = await fileContractLookup(
      wallet,
      contractAddress,
      abi,
      fileIdentifier
    );

    //mint NFT for owner
    mintToAddress(
      wallet,
      fileSmartContract,
      mintAbi,
      destinationWallet
    )
  })();
}

else if (command == "txLookUp") {
  (async() => {
      // NOTE: provide the public address, not private key
      const userWallet = process.argv[4];
      const apiKey = process.argv[5];

      //generate list of transactions using txhash
      const txList = await getAllUserTransactions(userWallet, apiKey);
      console.log("Transaction List:", txList);

      //filter ones only from registry contract
      const txTest = txList[0];
      await getTxReceipt(
        txTest,
        apiKey
      );

  })();
}


